@@ -9,27 +9,24 @@ EOF
 
 . "${TEST_SCRIPTS_DIR}/integration.bash"
 
-ctdb_test_init "$@"
+ctdb_test_init
 
 set -e
 
 cluster_is_healthy
 
-# Reset configuration
-ctdb_restart_when_done
-
 select_test_node_and_ips
 
 # Find interfaces on test node
 try_command_on_node $test_node "$CTDB ifaces -X"
-ifaces=$(awk -F'|' 'NR > 1 { print $2 }' "$outfile")
+ifaces=$(awk -F'|' 'NR > 1 { print $2 }' <<<"$out")
 echo "Node ${test_node} has interfaces: ${ifaces}"
 
 # Delete all IPs on each interface...  deleting IPs from one interface
 # can cause other interfaces to disappear, so we need to be careful...
 for i in $ifaces ; do
     try_command_on_node $test_node "$CTDB ifaces -X"
-    info=$(awk -F'|' -v iface="$i" '$2 == iface { print $0 }' "$outfile")
+    info=$(awk -F'|' -v iface="$i" '$2 == iface { print $0 }' <<<"$out")
 
     if [ -z "$info" ] ; then
 	echo "Interface ${i} missing... assuming already deleted!"
@@ -41,7 +38,7 @@ for i in $ifaces ; do
 
     try_command_on_node $test_node "$CTDB ip -v -X | tail -n +2"
     awk -F'|' -v i="$i" \
-	'$6 == i { print $2 }' "$outfile" |
+	'$6 == i { print $2 }' <<<"$out" |
     while read ip ; do
 	echo "  $ip"
 	try_command_on_node $test_node "$CTDB delip $ip"
@@ -49,8 +46,8 @@ for i in $ifaces ; do
     try_command_on_node $test_node "$CTDB ipreallocate"
 
     try_command_on_node $test_node "$CTDB ifaces -X"
-    info=$(awk -F'|' -v iface="$i" '$2 == iface { print $0 }' "$outfile")
-
+    info=$(awk -F'|' -v iface="$i" '$2 == iface { print $0 }' <<<"$out")
+    
     if [ -z "$info" ] ; then
 	echo "GOOD: Interface ${i} has been garbage collected"
     else
