@@ -25,7 +25,7 @@
 #include "includes.h"
 #include "lib/param/loadparm.h"
 
-const char toupper_ascii_fast_table[128] = {
+static const char toupper_ascii_fast_table[128] = {
 	0x0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa, 0xb, 0xc, 0xd, 0xe, 0xf,
 	0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
 	0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
@@ -170,32 +170,6 @@ bool trim_char(char *s,char cfront,char cback)
 	return ret;
 }
 
-/**
- Like strncpy but always null terminates. Make sure there is room!
- The variable n should always be one less than the available size.
-**/
-char *StrnCpy(char *dest,const char *src,size_t n)
-{
-	char *d = dest;
-
-	if (!dest) {
-		smb_panic("ERROR: NULL dest in StrnCpy");
-	}
-
-	if (!src) {
-		*dest = 0;
-		return(dest);
-	}
-
-	while (n-- && (*d = *src)) {
-		d++;
-		src++;
-	}
-
-	*d = 0;
-	return(dest);
-}
-
 /**
  Check if a string is part of a list.
 **/
@@ -276,6 +250,8 @@ char *talloc_string_sub2(TALLOC_CTX *mem_ctx, const char *src,
 				if (allow_trailing_dollar && (i == li - 1 )) {
 					break;
 				}
+
+				FALL_THROUGH;
 			case '`':
 			case '"':
 			case '\'':
@@ -287,6 +263,8 @@ char *talloc_string_sub2(TALLOC_CTX *mem_ctx, const char *src,
 					in[i] = '_';
 					break;
 				}
+
+				FALL_THROUGH;
 			default:
 				/* ok */
 				break;
@@ -559,14 +537,14 @@ bool strupper_m(char *s)
 	   (ie. they match for the first 128 chars) */
 
 	while (*s && !(((unsigned char)s[0]) & 0x80)) {
-		*s = toupper_ascii_fast((unsigned char)*s);
+		*s = toupper_ascii_fast_table[(unsigned char)s[0]];
 		s++;
 	}
 
 	if (!*s)
 		return true;
 
-	/* I assume that lowercased string takes the same number of bytes
+	/* I assume that uppercased string takes the same number of bytes
 	 * as source string even in multibyte encoding. (VIV) */
 	len = strlen(s) + 1;
 	ret = unix_strupper(s,len,s,len);
@@ -909,65 +887,6 @@ uint64_t conv_str_size(const char * str)
 	return lval;
 }
 
-/* Append an sprintf'ed string. Double buffer size on demand. Usable without
- * error checking in between. The indiation that something weird happened is
- * string==NULL */
-
-void sprintf_append(TALLOC_CTX *mem_ctx, char **string, ssize_t *len,
-		    size_t *bufsize, const char *fmt, ...)
-{
-	va_list ap;
-	char *newstr;
-	int ret;
-	bool increased;
-
-	/* len<0 is an internal marker that something failed */
-	if (*len < 0)
-		goto error;
-
-	if (*string == NULL) {
-		if (*bufsize == 0)
-			*bufsize = 128;
-
-		*string = talloc_array(mem_ctx, char, *bufsize);
-		if (*string == NULL)
-			goto error;
-	}
-
-	va_start(ap, fmt);
-	ret = vasprintf(&newstr, fmt, ap);
-	va_end(ap);
-
-	if (ret < 0)
-		goto error;
-
-	increased = false;
-
-	while ((*len)+ret >= *bufsize) {
-		increased = true;
-		*bufsize *= 2;
-		if (*bufsize >= (1024*1024*256))
-			goto error;
-	}
-
-	if (increased) {
-		*string = talloc_realloc(mem_ctx, *string, char,
-					       *bufsize);
-		if (*string == NULL) {
-			goto error;
-		}
-	}
-
-	StrnCpy((*string)+(*len), newstr, ret);
-	(*len) += ret;
-	free(newstr);
-	return;
-
- error:
-	*len = -1;
-	*string = NULL;
-}
-
 /*
  * asprintf into a string and strupper_m it after that.
  */
