@@ -52,10 +52,19 @@ char *ads_build_path(const char *realm, const char *sep, const char *field, int
 		return NULL;
 	}
 
-	strlcpy(ret,field, len);
+	if (strlcpy(ret,field, len) >= len) {
+		/* Truncate ! */
+		free(r);
+		free(ret);
+		return NULL;
+	}
 	p=strtok_r(r, sep, &saveptr);
 	if (p) {
-		strlcat(ret, p, len);
+		if (strlcat(ret, p, len) >= len) {
+			free(r);
+			free(ret);
+			return NULL;
+		}
 
 		while ((p=strtok_r(NULL, sep, &saveptr)) != NULL) {
 			int retval;
@@ -101,7 +110,11 @@ char *ads_build_domain(const char *dn)
 		return NULL;		
 	}	
 
-	strlower_m( dnsdomain );	
+	if (!strlower_m( dnsdomain )) {
+		SAFE_FREE(dnsdomain);
+		return NULL;
+	}
+
 	all_string_sub( dnsdomain, "dc=", "", 0);
 	all_string_sub( dnsdomain, ",", ".", 0 );
 
@@ -131,14 +144,6 @@ ADS_STRUCT *ads_init(const char *realm,
 	ads->server.workgroup = workgroup ? SMB_STRDUP(workgroup) : NULL;
 	ads->server.ldap_server = ldap_server? SMB_STRDUP(ldap_server) : NULL;
 
-	/* we need to know if this is a foreign realm */
-	if (realm && *realm && !strequal(lp_realm(), realm)) {
-		ads->server.foreign = 1;
-	}
-	if (workgroup && *workgroup && !strequal(lp_workgroup(), workgroup)) {
-		ads->server.foreign = 1;
-	}
-
 	/* the caller will own the memory by default */
 	ads->is_mine = 1;
 
@@ -149,9 +154,9 @@ ADS_STRUCT *ads_init(const char *realm,
 
 	ads->auth.flags = wrap_flags;
 
-	/* Start with a page size of 1000 when the connection is new,
+	/* Start with the configured page size when the connection is new,
 	 * we will drop it by half we get a timeout.   */
-	ads->config.ldap_page_size     = 1000;
+	ads->config.ldap_page_size     = lp_ldap_page_size();
 
 	return ads;
 }
@@ -179,7 +184,7 @@ void ads_destroy(ADS_STRUCT **ads)
 		bool is_mine;
 
 		is_mine = (*ads)->is_mine;
-#if HAVE_LDAP
+#ifdef HAVE_LDAP
 		ads_disconnect(*ads);
 #endif
 		SAFE_FREE((*ads)->server.realm);
@@ -190,6 +195,7 @@ void ads_destroy(ADS_STRUCT **ads)
 		SAFE_FREE((*ads)->auth.password);
 		SAFE_FREE((*ads)->auth.user_name);
 		SAFE_FREE((*ads)->auth.kdc_server);
+		SAFE_FREE((*ads)->auth.ccache_name);
 
 		SAFE_FREE((*ads)->config.realm);
 		SAFE_FREE((*ads)->config.bind_path);
