@@ -45,10 +45,12 @@
 #include "lib/util/sys_rw.h"
 #include "smbprofile.h"
 #include "modules/posixacl_xattr.h"
-#include "lib/pthreadpool/pthreadpool_tevent.h"
 
 #define DEFAULT_VOLFILE_SERVER "localhost"
-#define GLUSTER_NAME_MAX 255
+
+static int read_fd = -1;
+static int write_fd = -1;
+static struct tevent_fd *aio_read_event = NULL;
 
 /**
  * Helper to convert struct stat to struct stat_ex.
@@ -360,6 +362,11 @@ static int vfs_gluster_connect(struct vfs_handle_struct *handle,
 	 */
 	lp_do_parameter(SNUM(handle->conn), "shadow:mountpoint", "/");
 
+	/*
+	 * Unless we have an async implementation of getxattrat turn this off.
+	 */
+	lp_do_parameter(SNUM(handle->conn), "smbd:async dosmode", "false");
+
 done:
 	if (ret < 0) {
 		if (fs)
@@ -480,16 +487,12 @@ static DIR *vfs_gluster_opendir(struct vfs_handle_struct *handle,
 {
 	glfs_fd_t *fd;
 
-	START_PROFILE(syscall_opendir);
-
 	fd = glfs_opendir(handle->data, smb_fname->base_name);
 	if (fd == NULL) {
 		DEBUG(0, ("glfs_opendir(%s) failed: %s\n",
 			  smb_fname->base_name, strerror(errno)));
 	}
 
-	END_PROFILE(syscall_opendir);
-
 	return (DIR *) fd;
 }
 
@@ -524,13 +527,7 @@ static DIR *vfs_gluster_fdopendir(struct vfs_handle_struct *handle,
 
 static int vfs_gluster_closedir(struct vfs_handle_struct *handle, DIR *dirp)
 {
-	int ret;
-
-	START_PROFILE(syscall_closedir);
-	ret = glfs_closedir((void *)dirp);
-	END_PROFILE(syscall_closedir);
-
-	return ret;
+	return glfs_closedir((void *)dirp);
 }
 
 static struct dirent *vfs_gluster_readdir(struct vfs_handle_struct *handle,
@@ -541,7 +538,6 @@ static struct dirent *vfs_gluster_readdir(struct vfs_handle_struct *handle,
 	struct stat stat;
 	struct dirent *dirent = 0;
 
-	START_PROFILE(syscall_readdir);
 	if (sbuf != NULL) {
 		ret = glfs_readdirplus_r((void *)dirp, &stat, (void *)direntbuf,
 					 &dirent);
@@ -550,7 +546,6 @@ static struct dirent *vfs_gluster_readdir(struct vfs_handle_struct *handle,
 	}
 
 	if ((ret < 0) || (dirent == NULL)) {
-		END_PROFILE(syscall_readdir);
 		return NULL;
 	}
 
@@ -558,59 +553,36 @@ static struct dirent *vfs_gluster_readdir(struct vfs_handle_struct *handle,
 		smb_stat_ex_from_stat(sbuf, &stat);
 	}
 
-	END_PROFILE(syscall_readdir);
 	return dirent;
 }
 
 static long vfs_gluster_telldir(struct vfs_handle_struct *handle, DIR *dirp)
 {
-	long ret;
-
-	START_PROFILE(syscall_telldir);
-	ret = glfs_telldir((void *)dirp);
-	END_PROFILE(syscall_telldir);
-
-	return ret;
+	return glfs_telldir((void *)dirp);
 }
 
 static void vfs_gluster_seekdir(struct vfs_handle_struct *handle, DIR *dirp,
 				long offset)
 {
-	START_PROFILE(syscall_seekdir);
 	glfs_seekdir((void *)dirp, offset);
-	END_PROFILE(syscall_seekdir);
 }
 
 static void vfs_gluster_rewinddir(struct vfs_handle_struct *handle, DIR *dirp)
 {
-	START_PROFILE(syscall_rewinddir);
 	glfs_seekdir((void *)dirp, 0);
-	END_PROFILE(syscall_rewinddir);
 }
 
 static int vfs_gluster_mkdir(struct vfs_handle_struct *handle,
 			     const struct smb_filename *smb_fname,
 			     mode_t mode)
 {
-	int ret;
-
-	START_PROFILE(syscall_mkdir);
-	ret = glfs_mkdir(handle->data, smb_fname->base_name, mode);
-	END_PROFILE(syscall_mkdir);
-
-	return ret;
+	return glfs_mkdir(handle->data, smb_fname->base_name, mode);
 }
 
 static int vfs_gluster_rmdir(struct vfs_handle_struct *handle,
 			const struct smb_filename *smb_fname)
 {
-	int ret;
-
-	START_PROFILE(syscall_rmdir);
-	ret = glfs_rmdir(handle->data, smb_fname->base_name);
-	END_PROFILE(syscall_rmdir);
-
-	return ret;
+	return glfs_rmdir(handle->data, smb_fname->base_name);
 }
 
 static int vfs_gluster_open(struct vfs_handle_struct *handle,
@@ -620,8 +592,6 @@ static int vfs_gluster_open(struct vfs_handle_struct *handle,
 	glfs_fd_t *glfd;
 	glfs_fd_t **p_tmp;
 
-	START_PROFILE(syscall_open);
-
 	if (flags & O_DIRECTORY) {
 		glfd = glfs_opendir(handle->data, smb_fname->base_name);
 	} else if (flags & O_CREAT) {
@@ -632,13 +602,10 @@ static int vfs_gluster_open(struct vfs_handle_struct *handle,
 	}
 
 	if (glfd == NULL) {
-		END_PROFILE(syscall_open);
 		return -1;
 	}
 	p_tmp = VFS_ADD_FSP_EXTENSION(handle, fsp, glfs_fd_t *, NULL);
 	*p_tmp = glfd;
-
-	END_PROFILE(syscall_open);
 	/* An arbitrary value for error reporting, so you know its us. */
 	return 13371337;
 }
@@ -646,66 +613,219 @@ static int vfs_gluster_open(struct vfs_handle_struct *handle,
 static int vfs_gluster_close(struct vfs_handle_struct *handle,
 			     files_struct *fsp)
 {
-	int ret;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE(syscall_close);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_close);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
 	VFS_REMOVE_FSP_EXTENSION(handle, fsp);
-
-	ret = glfs_close(glfd);
-	END_PROFILE(syscall_close);
-
-	return ret;
+	return glfs_close(glfd);
 }
 
 static ssize_t vfs_gluster_pread(struct vfs_handle_struct *handle,
 				 files_struct *fsp, void *data, size_t n,
 				 off_t offset)
 {
-	ssize_t ret;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE_BYTES(syscall_pread, n);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE_BYTES(syscall_pread);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
 #ifdef HAVE_GFAPI_VER_7_6
-	ret = glfs_pread(glfd, data, n, offset, 0, NULL);
+	return glfs_pread(glfd, data, n, offset, 0, NULL);
 #else
-	ret = glfs_pread(glfd, data, n, offset, 0);
+	return glfs_pread(glfd, data, n, offset, 0);
 #endif
-	END_PROFILE_BYTES(syscall_pread);
-
-	return ret;
 }
 
-struct vfs_gluster_pread_state {
-	ssize_t ret;
-	glfs_fd_t *fd;
-	void *buf;
-	size_t count;
-	off_t offset;
+struct glusterfs_aio_state;
 
-	struct vfs_aio_state vfs_aio_state;
-	SMBPROFILE_BYTES_ASYNC_STATE(profile_bytes);
+struct glusterfs_aio_wrapper {
+	struct glusterfs_aio_state *state;
 };
 
-static void vfs_gluster_pread_do(void *private_data);
-static void vfs_gluster_pread_done(struct tevent_req *subreq);
-static int vfs_gluster_pread_state_destructor(struct vfs_gluster_pread_state *state);
+struct glusterfs_aio_state {
+	ssize_t ret;
+	struct tevent_req *req;
+	bool cancelled;
+	struct vfs_aio_state vfs_aio_state;
+	struct timespec start;
+};
+
+static int aio_wrapper_destructor(struct glusterfs_aio_wrapper *wrap)
+{
+	if (wrap->state != NULL) {
+		wrap->state->cancelled = true;
+	}
+
+	return 0;
+}
+
+/*
+ * This function is the callback that will be called on glusterfs
+ * threads once the async IO submitted is complete. To notify
+ * Samba of the completion we use a pipe based queue.
+ */
+#ifdef HAVE_GFAPI_VER_7_6
+static void aio_glusterfs_done(glfs_fd_t *fd, ssize_t ret,
+			       struct glfs_stat *prestat,
+			       struct glfs_stat *poststat,
+			       void *data)
+#else
+static void aio_glusterfs_done(glfs_fd_t *fd, ssize_t ret, void *data)
+#endif
+{
+	struct glusterfs_aio_state *state = NULL;
+	int sts = 0;
+	struct timespec end;
+
+	state = (struct glusterfs_aio_state *)data;
+
+	PROFILE_TIMESTAMP(&end);
+
+	if (ret < 0) {
+		state->ret = -1;
+		state->vfs_aio_state.error = errno;
+	} else {
+		state->ret = ret;
+	}
+	state->vfs_aio_state.duration = nsec_time_diff(&end, &state->start);
+
+	/*
+	 * Write the state pointer to glusterfs_aio_state to the
+	 * pipe, so we can call tevent_req_done() from the main thread,
+	 * because tevent_req_done() is not designed to be executed in
+	 * the multithread environment, so tevent_req_done() must be
+	 * executed from the smbd main thread.
+	 *
+	 * write(2) on pipes with sizes under _POSIX_PIPE_BUF
+	 * in size is atomic, without this, the use op pipes in this
+	 * code would not work.
+	 *
+	 * sys_write is a thin enough wrapper around write(2)
+	 * that we can trust it here.
+	 */
+
+	sts = sys_write(write_fd, &state, sizeof(struct glusterfs_aio_state *));
+	if (sts < 0) {
+		DEBUG(0,("\nWrite to pipe failed (%s)", strerror(errno)));
+	}
+
+	return;
+}
+
+/*
+ * Read each req off the pipe and process it.
+ */
+static void aio_tevent_fd_done(struct tevent_context *event_ctx,
+				struct tevent_fd *fde,
+				uint16_t flags, void *data)
+{
+	struct tevent_req *req = NULL;
+	struct glusterfs_aio_state *state = NULL;
+	int sts = 0;
+
+	/*
+	 * read(2) on pipes is atomic if the needed data is available
+	 * in the pipe, per SUS and POSIX.  Because we always write
+	 * to the pipe in sizeof(struct tevent_req *) chunks, we can
+	 * always read in those chunks, atomically.
+	 *
+	 * sys_read is a thin enough wrapper around read(2) that we
+	 * can trust it here.
+	 */
+
+	sts = sys_read(read_fd, &state, sizeof(struct glusterfs_aio_state *));
+
+	if (sts < 0) {
+		DEBUG(0,("\nRead from pipe failed (%s)", strerror(errno)));
+	}
+
+	/* if we've cancelled the op, there is no req, so just clean up. */
+	if (state->cancelled == true) {
+		TALLOC_FREE(state);
+		return;
+	}
+
+	req = state->req;
+
+	if (req) {
+		tevent_req_done(req);
+	}
+	return;
+}
+
+static bool init_gluster_aio(struct vfs_handle_struct *handle)
+{
+	int fds[2];
+	int ret = -1;
+
+	if (read_fd != -1) {
+		/*
+		 * Already initialized.
+		 */
+		return true;
+	}
+
+	ret = pipe(fds);
+	if (ret == -1) {
+		goto fail;
+	}
+
+	read_fd = fds[0];
+	write_fd = fds[1];
+
+	aio_read_event = tevent_add_fd(handle->conn->sconn->ev_ctx,
+					NULL,
+					read_fd,
+					TEVENT_FD_READ,
+					aio_tevent_fd_done,
+					NULL);
+	if (aio_read_event == NULL) {
+		goto fail;
+	}
+
+	return true;
+fail:
+	TALLOC_FREE(aio_read_event);
+	if (read_fd != -1) {
+		close(read_fd);
+		close(write_fd);
+		read_fd = -1;
+		write_fd = -1;
+	}
+	return false;
+}
+
+static struct glusterfs_aio_state *aio_state_create(TALLOC_CTX *mem_ctx)
+{
+	struct tevent_req *req = NULL;
+	struct glusterfs_aio_state *state = NULL;
+	struct glusterfs_aio_wrapper *wrapper = NULL;
+
+	req = tevent_req_create(mem_ctx, &wrapper, struct glusterfs_aio_wrapper);
+
+	if (req == NULL) {
+		return NULL;
+	}
+
+	state = talloc_zero(NULL, struct glusterfs_aio_state);
+
+	if (state == NULL) {
+		TALLOC_FREE(req);
+		return NULL;
+	}
+
+	talloc_set_destructor(wrapper, aio_wrapper_destructor);
+	state->cancelled = false;
+	state->req = req;
+
+	wrapper->state = state;
+
+	return state;
+}
 
 static struct tevent_req *vfs_gluster_pread_send(struct vfs_handle_struct
 						  *handle, TALLOC_CTX *mem_ctx,
@@ -714,128 +834,48 @@ static struct tevent_req *vfs_gluster_pread_send(struct vfs_handle_struct
 						  void *data, size_t n,
 						  off_t offset)
 {
-	struct vfs_gluster_pread_state *state;
-	struct tevent_req *req, *subreq;
-
+	struct glusterfs_aio_state *state = NULL;
+	struct tevent_req *req = NULL;
+	int ret = 0;
 	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
+
 	if (glfd == NULL) {
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return NULL;
 	}
 
-	req = tevent_req_create(mem_ctx, &state, struct vfs_gluster_pread_state);
-	if (req == NULL) {
+	state = aio_state_create(mem_ctx);
+
+	if (state == NULL) {
 		return NULL;
 	}
 
-	state->ret = -1;
-	state->fd = glfd;
-	state->buf = data;
-	state->count = n;
-	state->offset = offset;
+	req = state->req;
 
-	SMBPROFILE_BYTES_ASYNC_START(syscall_asys_pread, profile_p,
-				     state->profile_bytes, n);
-	SMBPROFILE_BYTES_ASYNC_SET_IDLE(state->profile_bytes);
-
-	subreq = pthreadpool_tevent_job_send(
-		state, ev, handle->conn->sconn->pool,
-		vfs_gluster_pread_do, state);
-	if (tevent_req_nomem(subreq, req)) {
+	if (!init_gluster_aio(handle)) {
+		tevent_req_error(req, EIO);
 		return tevent_req_post(req, ev);
 	}
-	tevent_req_set_callback(subreq, vfs_gluster_pread_done, req);
 
-	talloc_set_destructor(state, vfs_gluster_pread_state_destructor);
+	/*
+	 * aio_glusterfs_done and aio_tevent_fd_done()
+	 * use the raw tevent context. We need to use
+	 * tevent_req_defer_callback() in order to
+	 * use the event context we're started with.
+	 */
+	tevent_req_defer_callback(req, ev);
+
+	PROFILE_TIMESTAMP(&state->start);
+	ret = glfs_pread_async(glfd, data, n, offset, 0, aio_glusterfs_done,
+				state);
+	if (ret < 0) {
+		tevent_req_error(req, -ret);
+		return tevent_req_post(req, ev);
+	}
 
 	return req;
 }
 
-static void vfs_gluster_pread_do(void *private_data)
-{
-	struct vfs_gluster_pread_state *state = talloc_get_type_abort(
-		private_data, struct vfs_gluster_pread_state);
-	struct timespec start_time;
-	struct timespec end_time;
-
-	SMBPROFILE_BYTES_ASYNC_SET_BUSY(state->profile_bytes);
-
-	PROFILE_TIMESTAMP(&start_time);
-
-	do {
-#ifdef HAVE_GFAPI_VER_7_6
-		state->ret = glfs_pread(state->fd, state->buf, state->count,
-					state->offset, 0, NULL);
-#else
-		state->ret = glfs_pread(state->fd, state->buf, state->count,
-					state->offset, 0);
-#endif
-	} while ((state->ret == -1) && (errno == EINTR));
-
-	if (state->ret == -1) {
-		state->vfs_aio_state.error = errno;
-	}
-
-	PROFILE_TIMESTAMP(&end_time);
-
-	state->vfs_aio_state.duration = nsec_time_diff(&end_time, &start_time);
-
-	SMBPROFILE_BYTES_ASYNC_SET_IDLE(state->profile_bytes);
-}
-
-static int vfs_gluster_pread_state_destructor(struct vfs_gluster_pread_state *state)
-{
-	return -1;
-}
-
-static void vfs_gluster_pread_done(struct tevent_req *subreq)
-{
-	struct tevent_req *req = tevent_req_callback_data(
-		subreq, struct tevent_req);
-	struct vfs_gluster_pread_state *state = tevent_req_data(
-		req, struct vfs_gluster_pread_state);
-	int ret;
-
-	ret = pthreadpool_tevent_job_recv(subreq);
-	TALLOC_FREE(subreq);
-	SMBPROFILE_BYTES_ASYNC_END(state->profile_bytes);
-	talloc_set_destructor(state, NULL);
-	if (tevent_req_error(req, ret)) {
-		return;
-	}
-
-	tevent_req_done(req);
-}
-
-static ssize_t vfs_gluster_pread_recv(struct tevent_req *req,
-				      struct vfs_aio_state *vfs_aio_state)
-{
-	struct vfs_gluster_pread_state *state = tevent_req_data(
-		req, struct vfs_gluster_pread_state);
-
-	if (tevent_req_is_unix_error(req, &vfs_aio_state->error)) {
-		return -1;
-	}
-
-	*vfs_aio_state = state->vfs_aio_state;
-	return state->ret;
-}
-
-struct vfs_gluster_pwrite_state {
-	ssize_t ret;
-	glfs_fd_t *fd;
-	const void *buf;
-	size_t count;
-	off_t offset;
-
-	struct vfs_aio_state vfs_aio_state;
-	SMBPROFILE_BYTES_ASYNC_STATE(profile_bytes);
-};
-
-static void vfs_gluster_pwrite_do(void *private_data);
-static void vfs_gluster_pwrite_done(struct tevent_req *subreq);
-static int vfs_gluster_pwrite_state_destructor(struct vfs_gluster_pwrite_state *state);
-
 static struct tevent_req *vfs_gluster_pwrite_send(struct vfs_handle_struct
 						  *handle, TALLOC_CTX *mem_ctx,
 						  struct tevent_context *ev,
@@ -843,159 +883,105 @@ static struct tevent_req *vfs_gluster_pwrite_send(struct vfs_handle_struct
 						  const void *data, size_t n,
 						  off_t offset)
 {
-	struct tevent_req *req, *subreq;
-	struct vfs_gluster_pwrite_state *state;
-
+	struct glusterfs_aio_state *state = NULL;
+	struct tevent_req *req = NULL;
+	int ret = 0;
 	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
+
 	if (glfd == NULL) {
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return NULL;
 	}
 
-	req = tevent_req_create(mem_ctx, &state, struct vfs_gluster_pwrite_state);
-	if (req == NULL) {
+	state = aio_state_create(mem_ctx);
+
+	if (state == NULL) {
 		return NULL;
 	}
 
-	state->ret = -1;
-	state->fd = glfd;
-	state->buf = data;
-	state->count = n;
-	state->offset = offset;
+	req = state->req;
 
-	SMBPROFILE_BYTES_ASYNC_START(syscall_asys_pwrite, profile_p,
-				     state->profile_bytes, n);
-	SMBPROFILE_BYTES_ASYNC_SET_IDLE(state->profile_bytes);
-
-	subreq = pthreadpool_tevent_job_send(
-		state, ev, handle->conn->sconn->pool,
-		vfs_gluster_pwrite_do, state);
-	if (tevent_req_nomem(subreq, req)) {
+	if (!init_gluster_aio(handle)) {
+		tevent_req_error(req, EIO);
 		return tevent_req_post(req, ev);
 	}
-	tevent_req_set_callback(subreq, vfs_gluster_pwrite_done, req);
 
-	talloc_set_destructor(state, vfs_gluster_pwrite_state_destructor);
+	/*
+	 * aio_glusterfs_done and aio_tevent_fd_done()
+	 * use the raw tevent context. We need to use
+	 * tevent_req_defer_callback() in order to
+	 * use the event context we're started with.
+	 */
+	tevent_req_defer_callback(req, ev);
+
+	PROFILE_TIMESTAMP(&state->start);
+	ret = glfs_pwrite_async(glfd, data, n, offset, 0, aio_glusterfs_done,
+				state);
+	if (ret < 0) {
+		tevent_req_error(req, -ret);
+		return tevent_req_post(req, ev);
+	}
 
 	return req;
 }
 
-static void vfs_gluster_pwrite_do(void *private_data)
+static ssize_t vfs_gluster_recv(struct tevent_req *req,
+				struct vfs_aio_state *vfs_aio_state)
 {
-	struct vfs_gluster_pwrite_state *state = talloc_get_type_abort(
-		private_data, struct vfs_gluster_pwrite_state);
-	struct timespec start_time;
-	struct timespec end_time;
+	struct glusterfs_aio_wrapper *wrapper = NULL;
+	int ret = 0;
 
-	SMBPROFILE_BYTES_ASYNC_SET_BUSY(state->profile_bytes);
+	wrapper = tevent_req_data(req, struct glusterfs_aio_wrapper);
 
-	PROFILE_TIMESTAMP(&start_time);
-
-	do {
-#ifdef HAVE_GFAPI_VER_7_6
-		state->ret = glfs_pwrite(state->fd, state->buf, state->count,
-					 state->offset, 0, NULL, NULL);
-#else
-		state->ret = glfs_pwrite(state->fd, state->buf, state->count,
-					 state->offset, 0);
-#endif
-	} while ((state->ret == -1) && (errno == EINTR));
-
-	if (state->ret == -1) {
-		state->vfs_aio_state.error = errno;
+	if (wrapper == NULL) {
+		return -1;
 	}
 
-	PROFILE_TIMESTAMP(&end_time);
-
-	state->vfs_aio_state.duration = nsec_time_diff(&end_time, &start_time);
-
-	SMBPROFILE_BYTES_ASYNC_SET_IDLE(state->profile_bytes);
-}
-
-static int vfs_gluster_pwrite_state_destructor(struct vfs_gluster_pwrite_state *state)
-{
-	return -1;
-}
-
-static void vfs_gluster_pwrite_done(struct tevent_req *subreq)
-{
-	struct tevent_req *req = tevent_req_callback_data(
-		subreq, struct tevent_req);
-	struct vfs_gluster_pwrite_state *state = tevent_req_data(
-		req, struct vfs_gluster_pwrite_state);
-	int ret;
-
-	ret = pthreadpool_tevent_job_recv(subreq);
-	TALLOC_FREE(subreq);
-	SMBPROFILE_BYTES_ASYNC_END(state->profile_bytes);
-	talloc_set_destructor(state, NULL);
-	if (tevent_req_error(req, ret)) {
-		return;
+	if (wrapper->state == NULL) {
+		return -1;
 	}
 
-	tevent_req_done(req);
-}
-
-static ssize_t vfs_gluster_pwrite_recv(struct tevent_req *req,
-				       struct vfs_aio_state *vfs_aio_state)
-{
-	struct vfs_gluster_pwrite_state *state = tevent_req_data(
-		req, struct vfs_gluster_pwrite_state);
-
 	if (tevent_req_is_unix_error(req, &vfs_aio_state->error)) {
 		return -1;
 	}
 
-	*vfs_aio_state = state->vfs_aio_state;
+	*vfs_aio_state = wrapper->state->vfs_aio_state;
+	ret = wrapper->state->ret;
 
-	return state->ret;
+	/* Clean up the state, it is in a NULL context. */
+
+	TALLOC_FREE(wrapper->state);
+
+	return ret;
 }
 
 static ssize_t vfs_gluster_pwrite(struct vfs_handle_struct *handle,
 				  files_struct *fsp, const void *data,
 				  size_t n, off_t offset)
 {
-	ssize_t ret;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE_BYTES(syscall_pwrite, n);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE_BYTES(syscall_pwrite);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
 #ifdef HAVE_GFAPI_VER_7_6
-	ret = glfs_pwrite(glfd, data, n, offset, 0, NULL, NULL);
+	return glfs_pwrite(glfd, data, n, offset, 0, NULL, NULL);
 #else
-	ret = glfs_pwrite(glfd, data, n, offset, 0);
+	return glfs_pwrite(glfd, data, n, offset, 0);
 #endif
-	END_PROFILE_BYTES(syscall_pwrite);
-
-	return ret;
 }
 
 static off_t vfs_gluster_lseek(struct vfs_handle_struct *handle,
 			       files_struct *fsp, off_t offset, int whence)
 {
-	off_t ret = 0;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE(syscall_lseek);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_lseek);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
-	ret = glfs_lseek(glfd, offset, whence);
-	END_PROFILE(syscall_lseek);
-
-	return ret;
+	return glfs_lseek(glfd, offset, whence);
 }
 
 static ssize_t vfs_gluster_sendfile(struct vfs_handle_struct *handle, int tofd,
@@ -1019,128 +1005,62 @@ static int vfs_gluster_rename(struct vfs_handle_struct *handle,
 			      const struct smb_filename *smb_fname_src,
 			      const struct smb_filename *smb_fname_dst)
 {
-	int ret;
-
-	START_PROFILE(syscall_rename);
-	ret = glfs_rename(handle->data, smb_fname_src->base_name,
-			  smb_fname_dst->base_name);
-	END_PROFILE(syscall_rename);
-
-	return ret;
+	return glfs_rename(handle->data, smb_fname_src->base_name,
+			   smb_fname_dst->base_name);
 }
 
-struct vfs_gluster_fsync_state {
-	ssize_t ret;
-	glfs_fd_t *fd;
-
-	struct vfs_aio_state vfs_aio_state;
-	SMBPROFILE_BASIC_ASYNC_STATE(profile_bytes);
-};
-
-static void vfs_gluster_fsync_do(void *private_data);
-static void vfs_gluster_fsync_done(struct tevent_req *subreq);
-static int vfs_gluster_fsync_state_destructor(struct vfs_gluster_fsync_state *state);
-
 static struct tevent_req *vfs_gluster_fsync_send(struct vfs_handle_struct
 						 *handle, TALLOC_CTX *mem_ctx,
 						 struct tevent_context *ev,
 						 files_struct *fsp)
 {
-	struct tevent_req *req, *subreq;
-	struct vfs_gluster_fsync_state *state;
-
+	struct tevent_req *req = NULL;
+	struct glusterfs_aio_state *state = NULL;
+	int ret = 0;
 	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
+
 	if (glfd == NULL) {
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return NULL;
 	}
 
-	req = tevent_req_create(mem_ctx, &state, struct vfs_gluster_fsync_state);
-	if (req == NULL) {
+	state = aio_state_create(mem_ctx);
+
+	if (state == NULL) {
 		return NULL;
 	}
 
-	state->ret = -1;
-	state->fd = glfd;
+	req = state->req;
 
-	SMBPROFILE_BASIC_ASYNC_START(syscall_asys_fsync, profile_p,
-                                     state->profile_bytes);
-
-	subreq = pthreadpool_tevent_job_send(
-		state, ev, handle->conn->sconn->pool, vfs_gluster_fsync_do, state);
-	if (tevent_req_nomem(subreq, req)) {
+	if (!init_gluster_aio(handle)) {
+		tevent_req_error(req, EIO);
 		return tevent_req_post(req, ev);
 	}
-	tevent_req_set_callback(subreq, vfs_gluster_fsync_done, req);
 
-	talloc_set_destructor(state, vfs_gluster_fsync_state_destructor);
+	/*
+	 * aio_glusterfs_done and aio_tevent_fd_done()
+	 * use the raw tevent context. We need to use
+	 * tevent_req_defer_callback() in order to
+	 * use the event context we're started with.
+	 */
+	tevent_req_defer_callback(req, ev);
 
+	PROFILE_TIMESTAMP(&state->start);
+	ret = glfs_fsync_async(glfd, aio_glusterfs_done, state);
+	if (ret < 0) {
+		tevent_req_error(req, -ret);
+		return tevent_req_post(req, ev);
+	}
 	return req;
 }
 
-static void vfs_gluster_fsync_do(void *private_data)
-{
-	struct vfs_gluster_fsync_state *state = talloc_get_type_abort(
-		private_data, struct vfs_gluster_fsync_state);
-	struct timespec start_time;
-	struct timespec end_time;
-
-	PROFILE_TIMESTAMP(&start_time);
-
-	do {
-#ifdef HAVE_GFAPI_VER_7_6
-		state->ret = glfs_fsync(state->fd, NULL, NULL);
-#else
-		state->ret = glfs_fsync(state->fd);
-#endif
-	} while ((state->ret == -1) && (errno == EINTR));
-
-	if (state->ret == -1) {
-		state->vfs_aio_state.error = errno;
-	}
-
-	PROFILE_TIMESTAMP(&end_time);
-
-	state->vfs_aio_state.duration = nsec_time_diff(&end_time, &start_time);
-}
-
-static int vfs_gluster_fsync_state_destructor(struct vfs_gluster_fsync_state *state)
-{
-	return -1;
-}
-
-static void vfs_gluster_fsync_done(struct tevent_req *subreq)
-{
-	struct tevent_req *req = tevent_req_callback_data(
-		subreq, struct tevent_req);
-	struct vfs_gluster_fsync_state *state = tevent_req_data(
-		req, struct vfs_gluster_fsync_state);
-	int ret;
-
-	ret = pthreadpool_tevent_job_recv(subreq);
-	TALLOC_FREE(subreq);
-	SMBPROFILE_BASIC_ASYNC_END(state->profile_bytes);
-	talloc_set_destructor(state, NULL);
-
-	if (tevent_req_error(req, ret)) {
-		return;
-	}
-
-	tevent_req_done(req);
-}
-
 static int vfs_gluster_fsync_recv(struct tevent_req *req,
 				  struct vfs_aio_state *vfs_aio_state)
 {
-	struct vfs_gluster_fsync_state *state = tevent_req_data(
-		req, struct vfs_gluster_fsync_state);
-
-	if (tevent_req_is_unix_error(req, &vfs_aio_state->error)) {
-		return -1;
-	}
-
-	*vfs_aio_state = state->vfs_aio_state;
-	return state->ret;
+	/*
+	 * Use implicit conversion ssize_t->int
+	 */
+	return vfs_gluster_recv(req, vfs_aio_state);
 }
 
 static int vfs_gluster_stat(struct vfs_handle_struct *handle,
@@ -1149,7 +1069,6 @@ static int vfs_gluster_stat(struct vfs_handle_struct *handle,
 	struct stat st;
 	int ret;
 
-	START_PROFILE(syscall_stat);
 	ret = glfs_stat(handle->data, smb_fname->base_name, &st);
 	if (ret == 0) {
 		smb_stat_ex_from_stat(&smb_fname->st, &st);
@@ -1158,8 +1077,6 @@ static int vfs_gluster_stat(struct vfs_handle_struct *handle,
 		DEBUG(0, ("glfs_stat(%s) failed: %s\n",
 			  smb_fname->base_name, strerror(errno)));
 	}
-	END_PROFILE(syscall_stat);
-
 	return ret;
 }
 
@@ -1168,13 +1085,9 @@ static int vfs_gluster_fstat(struct vfs_handle_struct *handle,
 {
 	struct stat st;
 	int ret;
-	glfs_fd_t *glfd = NULL;
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 
-	START_PROFILE(syscall_fstat);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_fstat);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
@@ -1187,8 +1100,6 @@ static int vfs_gluster_fstat(struct vfs_handle_struct *handle,
 		DEBUG(0, ("glfs_fstat(%d) failed: %s\n",
 			  fsp->fh->fd, strerror(errno)));
 	}
-	END_PROFILE(syscall_fstat);
-
 	return ret;
 }
 
@@ -1198,7 +1109,6 @@ static int vfs_gluster_lstat(struct vfs_handle_struct *handle,
 	struct stat st;
 	int ret;
 
-	START_PROFILE(syscall_lstat);
 	ret = glfs_lstat(handle->data, smb_fname->base_name, &st);
 	if (ret == 0) {
 		smb_stat_ex_from_stat(&smb_fname->st, &st);
@@ -1207,8 +1117,6 @@ static int vfs_gluster_lstat(struct vfs_handle_struct *handle,
 		DEBUG(0, ("glfs_lstat(%s) failed: %s\n",
 			  smb_fname->base_name, strerror(errno)));
 	}
-	END_PROFILE(syscall_lstat);
-
 	return ret;
 }
 
@@ -1216,59 +1124,33 @@ static uint64_t vfs_gluster_get_alloc_size(struct vfs_handle_struct *handle,
 					   files_struct *fsp,
 					   const SMB_STRUCT_STAT *sbuf)
 {
-	uint64_t ret;
-
-	START_PROFILE(syscall_get_alloc_size);
-	ret = sbuf->st_ex_blocks * 512;
-	END_PROFILE(syscall_get_alloc_size);
-
-	return ret;
+	return sbuf->st_ex_blocks * 512;
 }
 
 static int vfs_gluster_unlink(struct vfs_handle_struct *handle,
 			      const struct smb_filename *smb_fname)
 {
-	int ret;
-
-	START_PROFILE(syscall_unlink);
-	ret = glfs_unlink(handle->data, smb_fname->base_name);
-	END_PROFILE(syscall_unlink);
-
-	return ret;
+	return glfs_unlink(handle->data, smb_fname->base_name);
 }
 
 static int vfs_gluster_chmod(struct vfs_handle_struct *handle,
 				const struct smb_filename *smb_fname,
 				mode_t mode)
 {
-	int ret;
-
-	START_PROFILE(syscall_chmod);
-	ret = glfs_chmod(handle->data, smb_fname->base_name, mode);
-	END_PROFILE(syscall_chmod);
-
-	return ret;
+	return glfs_chmod(handle->data, smb_fname->base_name, mode);
 }
 
 static int vfs_gluster_fchmod(struct vfs_handle_struct *handle,
 			      files_struct *fsp, mode_t mode)
 {
-	int ret;
-	glfs_fd_t *glfd = NULL;
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 
-	START_PROFILE(syscall_fchmod);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_fchmod);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
-	ret = glfs_fchmod(glfd, mode);
-	END_PROFILE(syscall_fchmod);
-
-	return ret;
+	return glfs_fchmod(glfd, mode);
 }
 
 static int vfs_gluster_chown(struct vfs_handle_struct *handle,
@@ -1276,34 +1158,19 @@ static int vfs_gluster_chown(struct vfs_handle_struct *handle,
 			uid_t uid,
 			gid_t gid)
 {
-	int ret;
-
-	START_PROFILE(syscall_chown);
-	ret = glfs_chown(handle->data, smb_fname->base_name, uid, gid);
-	END_PROFILE(syscall_chown);
-
-	return ret;
+	return glfs_chown(handle->data, smb_fname->base_name, uid, gid);
 }
 
 static int vfs_gluster_fchown(struct vfs_handle_struct *handle,
 			      files_struct *fsp, uid_t uid, gid_t gid)
 {
-	int ret;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE(syscall_fchown);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_fchown);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
-	ret = glfs_fchown(glfd, uid, gid);
-	END_PROFILE(syscall_fchown);
-
-	return ret;
+	return glfs_fchown(glfd, uid, gid);
 }
 
 static int vfs_gluster_lchown(struct vfs_handle_struct *handle,
@@ -1311,25 +1178,13 @@ static int vfs_gluster_lchown(struct vfs_handle_struct *handle,
 			uid_t uid,
 			gid_t gid)
 {
-	int ret;
-
-	START_PROFILE(syscall_lchown);
-	ret = glfs_lchown(handle->data, smb_fname->base_name, uid, gid);
-	END_PROFILE(syscall_lchown);
-
-	return ret;
+	return glfs_lchown(handle->data, smb_fname->base_name, uid, gid);
 }
 
 static int vfs_gluster_chdir(struct vfs_handle_struct *handle,
 			const struct smb_filename *smb_fname)
 {
-	int ret;
-
-	START_PROFILE(syscall_chdir);
-	ret = glfs_chdir(handle->data, smb_fname->base_name);
-	END_PROFILE(syscall_chdir);
-
-	return ret;
+	return glfs_chdir(handle->data, smb_fname->base_name);
 }
 
 static struct smb_filename *vfs_gluster_getwd(struct vfs_handle_struct *handle,
@@ -1339,17 +1194,12 @@ static struct smb_filename *vfs_gluster_getwd(struct vfs_handle_struct *handle,
 	char *ret;
 	struct smb_filename *smb_fname = NULL;
 
-	START_PROFILE(syscall_getwd);
-
 	cwd = SMB_CALLOC_ARRAY(char, PATH_MAX);
 	if (cwd == NULL) {
-		END_PROFILE(syscall_getwd);
 		return NULL;
 	}
 
 	ret = glfs_getcwd(handle->data, cwd, PATH_MAX - 1);
-	END_PROFILE(syscall_getwd);
-
 	if (ret == NULL) {
 		SAFE_FREE(cwd);
 		return NULL;
@@ -1367,11 +1217,8 @@ static int vfs_gluster_ntimes(struct vfs_handle_struct *handle,
 			      const struct smb_filename *smb_fname,
 			      struct smb_file_time *ft)
 {
-	int ret = -1;
 	struct timespec times[2];
 
-	START_PROFILE(syscall_ntimes);
-
 	if (null_timespec(ft->atime)) {
 		times[0].tv_sec = smb_fname->st.st_ex_atime.tv_sec;
 		times[0].tv_nsec = smb_fname->st.st_ex_atime.tv_nsec;
@@ -1392,39 +1239,26 @@ static int vfs_gluster_ntimes(struct vfs_handle_struct *handle,
 			      &smb_fname->st.st_ex_atime) == 0) &&
 	    (timespec_compare(&times[1],
 			      &smb_fname->st.st_ex_mtime) == 0)) {
-		END_PROFILE(syscall_ntimes);
 		return 0;
 	}
 
-	ret = glfs_utimens(handle->data, smb_fname->base_name, times);
-	END_PROFILE(syscall_ntimes);
-
-	return ret;
+	return glfs_utimens(handle->data, smb_fname->base_name, times);
 }
 
 static int vfs_gluster_ftruncate(struct vfs_handle_struct *handle,
 				 files_struct *fsp, off_t offset)
 {
-	int ret;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE(syscall_ftruncate);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_ftruncate);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
 
 #ifdef HAVE_GFAPI_VER_7_6
-	ret = glfs_ftruncate(glfd, offset, NULL, NULL);
+	return glfs_ftruncate(glfd, offset, NULL, NULL);
 #else
-	ret = glfs_ftruncate(glfd, offset);
+	return glfs_ftruncate(glfd, offset);
 #endif
-	END_PROFILE(syscall_ftruncate);
-
-	return ret;
 }
 
 static int vfs_gluster_fallocate(struct vfs_handle_struct *handle,
@@ -1432,16 +1266,10 @@ static int vfs_gluster_fallocate(struct vfs_handle_struct *handle,
 				 uint32_t mode,
 				 off_t offset, off_t len)
 {
-	int ret;
 #ifdef HAVE_GFAPI_VER_6
-	glfs_fd_t *glfd = NULL;
 	int keep_size, punch_hole;
-
-	START_PROFILE(syscall_fallocate);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_fallocate);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return -1;
 	}
@@ -1451,22 +1279,19 @@ static int vfs_gluster_fallocate(struct vfs_handle_struct *handle,
 
 	mode &= ~(VFS_FALLOCATE_FL_KEEP_SIZE|VFS_FALLOCATE_FL_PUNCH_HOLE);
 	if (mode != 0) {
-		END_PROFILE(syscall_fallocate);
 		errno = ENOTSUP;
 		return -1;
 	}
 
 	if (punch_hole) {
-		ret = glfs_discard(glfd, offset, len);
+		return glfs_discard(glfd, offset, len);
 	}
 
-	ret = glfs_fallocate(glfd, keep_size, offset, len);
-	END_PROFILE(syscall_fallocate);
+	return glfs_fallocate(glfd, keep_size, offset, len);
 #else
 	errno = ENOTSUP;
-	ret = -1;
+	return -1;
 #endif
-	return ret;
 }
 
 static struct smb_filename *vfs_gluster_realpath(struct vfs_handle_struct *handle,
@@ -1475,13 +1300,9 @@ static struct smb_filename *vfs_gluster_realpath(struct vfs_handle_struct *handl
 {
 	char *result = NULL;
 	struct smb_filename *result_fname = NULL;
-	char *resolved_path = NULL;
+	char *resolved_path = SMB_MALLOC_ARRAY(char, PATH_MAX+1);
 
-	START_PROFILE(syscall_realpath);
-
-	resolved_path = SMB_MALLOC_ARRAY(char, PATH_MAX+1);
 	if (resolved_path == NULL) {
-		END_PROFILE(syscall_realpath);
 		errno = ENOMEM;
 		return NULL;
 	}
@@ -1494,8 +1315,6 @@ static struct smb_filename *vfs_gluster_realpath(struct vfs_handle_struct *handl
 	}
 
 	SAFE_FREE(resolved_path);
-	END_PROFILE(syscall_realpath);
-
 	return result_fname;
 }
 
@@ -1505,16 +1324,10 @@ static bool vfs_gluster_lock(struct vfs_handle_struct *handle,
 {
 	struct flock flock = { 0, };
 	int ret;
-	glfs_fd_t *glfd = NULL;
-	bool ok = false;
-
-	START_PROFILE(syscall_fcntl_lock);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
 		DBG_ERR("Failed to fetch gluster fd\n");
-		ok = false;
-		goto out;
+		return false;
 	}
 
 	flock.l_type = type;
@@ -1529,25 +1342,17 @@ static bool vfs_gluster_lock(struct vfs_handle_struct *handle,
 		/* lock query, true if someone else has locked */
 		if ((ret != -1) &&
 		    (flock.l_type != F_UNLCK) &&
-		    (flock.l_pid != 0) && (flock.l_pid != getpid())) {
-			ok = true;
-			goto out;
-		}
+		    (flock.l_pid != 0) && (flock.l_pid != getpid()))
+			return true;
 		/* not me */
-		ok = false;
-		goto out;
+		return false;
 	}
 
 	if (ret == -1) {
-		ok = false;
-		goto out;
+		return false;
 	}
 
-	ok = true;
-out:
-	END_PROFILE(syscall_fcntl_lock);
-
-	return ok;
+	return true;
 }
 
 static int vfs_gluster_kernel_flock(struct vfs_handle_struct *handle,
@@ -1571,13 +1376,8 @@ static bool vfs_gluster_getlock(struct vfs_handle_struct *handle,
 {
 	struct flock flock = { 0, };
 	int ret;
-	glfs_fd_t *glfd = NULL;
-
-	START_PROFILE(syscall_fcntl_getlock);
-
-	glfd = vfs_gluster_fetch_glfd(handle, fsp);
+	glfs_fd_t *glfd = vfs_gluster_fetch_glfd(handle, fsp);
 	if (glfd == NULL) {
-		END_PROFILE(syscall_fcntl_getlock);
 		DBG_ERR("Failed to fetch gluster fd\n");
 		return false;
 	}
@@ -1591,7 +1391,6 @@ static bool vfs_gluster_getlock(struct vfs_handle_struct *handle,
 	ret = glfs_posix_lock(glfd, F_GETLK, &flock);
 
 	if (ret == -1) {
-		END_PROFILE(syscall_fcntl_getlock);
 		return false;
 	}
 
@@ -1599,7 +1398,6 @@ static bool vfs_gluster_getlock(struct vfs_handle_struct *handle,
 	*poffset = flock.l_start;
 	*pcount = flock.l_len;
 	*ppid = flock.l_pid;
-	END_PROFILE(syscall_fcntl_getlock);
 
 	return true;
 }
@@ -1608,15 +1406,9 @@ static int vfs_gluster_symlink(struct vfs_handle_struct *handle,
 				const char *link_target,
 				const struct smb_filename *new_smb_fname)
 {
-	int ret;
-
-	START_PROFILE(syscall_symlink);
-	ret = glfs_symlink(handle->data,
+	return glfs_symlink(handle->data,
 			link_target,
 			new_smb_fname->base_name);
-	END_PROFILE(syscall_symlink);
-
-	return ret;
 }
 
 static int vfs_gluster_readlink(struct vfs_handle_struct *handle,
@@ -1624,28 +1416,16 @@ static int vfs_gluster_readlink(struct vfs_handle_struct *handle,
 				char *buf,
 				size_t bufsiz)
 {
-	int ret;
-
-	START_PROFILE(syscall_readlink);
-	ret = glfs_readlink(handle->data, smb_fname->base_name, buf, bufsiz);
-	END_PROFILE(syscall_readlink);
-
-	return ret;
+	return glfs_readlink(handle->data, smb_fname->base_name, buf, bufsiz);
 }
 
 static int vfs_gluster_link(struct vfs_handle_struct *handle,
 				const struct smb_filename *old_smb_fname,
 				const struct smb_filename *new_smb_fname)
 {
-	int ret;
-
-	START_PROFILE(syscall_link);
-	ret = glfs_link(handle->data,
+	return glfs_link(handle->data,
 			old_smb_fname->base_name,
 			new_smb_fname->base_name);
-	END_PROFILE(syscall_link);
-
-	return ret;
 }
 
 static int vfs_gluster_mknod(struct vfs_handle_struct *handle,
@@ -1653,13 +1433,7 @@ static int vfs_gluster_mknod(struct vfs_handle_struct *handle,
 				mode_t mode,
 				SMB_DEV_T dev)
 {
-	int ret;
-
-	START_PROFILE(syscall_mknod);
-	ret = glfs_mknod(handle->data, smb_fname->base_name, mode, dev);
-	END_PROFILE(syscall_mknod);
-
-	return ret;
+	return glfs_mknod(handle->data, smb_fname->base_name, mode, dev);
 }
 
 static int vfs_gluster_chflags(struct vfs_handle_struct *handle,
@@ -1675,22 +1449,21 @@ static int vfs_gluster_get_real_filename(struct vfs_handle_struct *handle,
 					 TALLOC_CTX *mem_ctx, char **found_name)
 {
 	int ret;
-	char key_buf[GLUSTER_NAME_MAX + 64];
-	char val_buf[GLUSTER_NAME_MAX + 1];
+	char key_buf[NAME_MAX + 64];
+	char val_buf[NAME_MAX + 1];
 
-	if (strlen(name) >= GLUSTER_NAME_MAX) {
+	if (strlen(name) >= NAME_MAX) {
 		errno = ENAMETOOLONG;
 		return -1;
 	}
 
-	snprintf(key_buf, GLUSTER_NAME_MAX + 64,
+	snprintf(key_buf, NAME_MAX + 64,
 		 "glusterfs.get_real_filename:%s", name);
 
-	ret = glfs_getxattr(handle->data, path, key_buf, val_buf,
-			    GLUSTER_NAME_MAX + 1);
+	ret = glfs_getxattr(handle->data, path, key_buf, val_buf, NAME_MAX + 1);
 	if (ret == -1) {
 		if (errno == ENOATTR) {
-			errno = ENOENT;
+			errno = EOPNOTSUPP;
 		}
 		return -1;
 	}
@@ -1718,7 +1491,7 @@ static ssize_t vfs_gluster_getxattr(struct vfs_handle_struct *handle,
 				size_t size)
 {
 	return glfs_getxattr(handle->data, smb_fname->base_name,
-			     name, value, size);
+			name, value, size);
 }
 
 static ssize_t vfs_gluster_fgetxattr(struct vfs_handle_struct *handle,
@@ -1836,10 +1609,10 @@ static struct vfs_fn_pointers glusterfs_fns = {
 	.close_fn = vfs_gluster_close,
 	.pread_fn = vfs_gluster_pread,
 	.pread_send_fn = vfs_gluster_pread_send,
-	.pread_recv_fn = vfs_gluster_pread_recv,
+	.pread_recv_fn = vfs_gluster_recv,
 	.pwrite_fn = vfs_gluster_pwrite,
 	.pwrite_send_fn = vfs_gluster_pwrite_send,
-	.pwrite_recv_fn = vfs_gluster_pwrite_recv,
+	.pwrite_recv_fn = vfs_gluster_recv,
 	.lseek_fn = vfs_gluster_lseek,
 	.sendfile_fn = vfs_gluster_sendfile,
 	.recvfile_fn = vfs_gluster_recvfile,
@@ -1902,6 +1675,8 @@ static struct vfs_fn_pointers glusterfs_fns = {
 
 	/* EA Operations */
 	.getxattr_fn = vfs_gluster_getxattr,
+	.getxattrat_send_fn = vfs_not_implemented_getxattrat_send,
+	.getxattrat_recv_fn = vfs_not_implemented_getxattrat_recv,
 	.fgetxattr_fn = vfs_gluster_fgetxattr,
 	.listxattr_fn = vfs_gluster_listxattr,
 	.flistxattr_fn = vfs_gluster_flistxattr,
