@@ -35,6 +35,7 @@
 #include "system/filesys.h"
 #include "popt_common.h"
 #include "lib/param/loadparm.h"
+#include "cmdline_contexts.h"
 
 #include <regex.h>
 
@@ -665,11 +666,11 @@ static void do_per_share_checks(int s)
 
 	smb_init_locale();
 	/*
-	 * Set the default debug level to 2.
+	 * Set the default debug level to 1.
 	 * Allow it to be overridden by the command line,
 	 * not by smb.conf.
 	 */
-	lp_set_cmdline("log level", "2");
+	lp_set_cmdline("log level", "1");
 
 	pc = poptGetContext(NULL, argc, argv, long_options,
 			    POPT_CONTEXT_KEEP_FIRST);
@@ -698,6 +699,8 @@ static void do_per_share_checks(int s)
 		goto done;
 	}
 
+	cmdline_messaging_context(config_file);
+
 	fprintf(stderr,"Load smb config files from %s\n",config_file);
 
 	if (!lp_load_with_registry_shares(config_file)) {
