@@ -29,6 +29,8 @@
 #include "librpc/gen_ndr/ndr_lsa_c.h"
 #include "libcli/security/dom_sid.h"
 #include "passdb/lookup_sid.h" /* only for LOOKUP_NAME_NO_NSS flag */
+#include "librpc/gen_ndr/ndr_irpc.h"
+#include "librpc/gen_ndr/ndr_netlogon.h"
 
 struct wb_irpc_forward_state {
 	struct irpc_message *msg;
@@ -127,7 +129,7 @@ static NTSTATUS wb_irpc_DsrUpdateReadOnlyServerDnsRecords(struct irpc_message *m
 	DEBUG(5, ("wb_irpc_DsrUpdateReadOnlyServerDnsRecords called\n"));
 
 	return wb_irpc_forward_rpc_call(msg, msg,
-					server_event_context(),
+					global_event_context(),
 					req, NDR_WINBIND_DSRUPDATEREADONLYSERVERDNSRECORDS,
 					"winbind_DsrUpdateReadOnlyServerDnsRecords",
 					domain, IRPC_CALL_TIMEOUT);
@@ -210,7 +212,7 @@ static NTSTATUS wb_irpc_SamLogon(struct irpc_message *msg,
 	DEBUG(5, ("wb_irpc_SamLogon called\n"));
 
 	return wb_irpc_forward_rpc_call(msg, msg,
-					server_event_context(),
+					global_event_context(),
 					req, NDR_WINBIND_SAMLOGON,
 					"winbind_SamLogon",
 					domain, IRPC_CALL_TIMEOUT);
@@ -270,7 +272,7 @@ static NTSTATUS wb_irpc_LogonControl(struct irpc_message *msg,
 
 	TALLOC_FREE(frame);
 	return wb_irpc_forward_rpc_call(msg, msg,
-					server_event_context(),
+					global_event_context(),
 					req, NDR_WINBIND_LOGONCONTROL,
 					"winbind_LogonControl",
 					domain, 45 /* timeout */);
@@ -310,7 +312,7 @@ static NTSTATUS wb_irpc_GetForestTrustInformation(struct irpc_message *msg,
 	DEBUG(5, ("wb_irpc_GetForestTrustInformation called\n"));
 
 	return wb_irpc_forward_rpc_call(msg, msg,
-					server_event_context(),
+					global_event_context(),
 					req, NDR_WINBIND_GETFORESTTRUSTINFORMATION,
 					"winbind_GetForestTrustInformation",
 					domain, 45 /* timeout */);
@@ -328,7 +330,7 @@ static NTSTATUS wb_irpc_SendToSam(struct irpc_message *msg,
 	DEBUG(5, ("wb_irpc_SendToSam called\n"));
 
 	return wb_irpc_forward_rpc_call(msg, msg,
-					server_event_context(),
+					global_event_context(),
 					req, NDR_WINBIND_SENDTOSAM,
 					"winbind_SendToSam",
 					domain, IRPC_CALL_TIMEOUT);
@@ -394,7 +396,7 @@ static NTSTATUS wb_irpc_lsa_LookupSids3_call(struct irpc_message *msg,
 	}
 
 	subreq = wb_lookupsids_send(msg,
-				    server_event_context(),
+				    global_event_context(),
 				    sids, req->in.sids->num_sids);
 	if (subreq == NULL) {
 		return NT_STATUS_NO_MEMORY;
@@ -571,7 +573,7 @@ static NTSTATUS wb_irpc_lsa_LookupNames4_call(struct irpc_message *msg,
 		}
 
 		subreq = wb_lookupname_send(msg,
-					    server_event_context(),
+					    global_event_context(),
 					    nstate->namespace,
 					    nstate->domain,
 					    nstate->name,
@@ -600,6 +602,7 @@ static void wb_irpc_lsa_LookupNames4_done(struct tevent_req *subreq)
 	struct wb_irpc_lsa_LookupNames4_state *state =
 		talloc_get_type_abort(nstate->state,
 		struct wb_irpc_lsa_LookupNames4_state);
+	struct dom_sid_buf buf;
 	NTSTATUS status;
 
 	SMB_ASSERT(state->num_pending > 0);
@@ -617,7 +620,8 @@ static void wb_irpc_lsa_LookupNames4_done(struct tevent_req *subreq)
 				   &nstate->authority_sid, NULL);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("dom_sid_split_rid(%s) failed - %s\n",
-			 sid_string_dbg(&nstate->sid), nt_errstr(status));
+			dom_sid_str_buf(&nstate->sid, &buf),
+			nt_errstr(status));
 		irpc_send_reply(state->msg, status);
 		return;
 	}
@@ -628,7 +632,8 @@ static void wb_irpc_lsa_LookupNames4_done(struct tevent_req *subreq)
 					 &state->num_domain_sids);
 	if (!NT_STATUS_IS_OK(status)) {
 		DBG_ERR("add_sid_to_array_unique(%s) failed - %s\n",
-			 sid_string_dbg(nstate->authority_sid), nt_errstr(status));
+			dom_sid_str_buf(nstate->authority_sid, &buf),
+			nt_errstr(status));
 		irpc_send_reply(state->msg, status);
 		return;
 	}
@@ -645,7 +650,7 @@ static void wb_irpc_lsa_LookupNames4_done(struct tevent_req *subreq)
 	 * to get a good lsa_RefDomainList
 	 */
 	subreq = wb_lookupsids_send(state,
-				    server_event_context(),
+				    global_event_context(),
 				    state->domain_sids,
 				    state->num_domain_sids);
 	if (subreq == NULL) {
@@ -729,6 +734,66 @@ static void wb_irpc_lsa_LookupNames4_domains_done(struct tevent_req *subreq)
 	return;
 }
 
+struct wb_irpc_GetDCName_state {
+	struct irpc_message *msg;
+	struct wbint_DsGetDcName *req;
+};
+
+static void wb_irpc_GetDCName_done(struct tevent_req *subreq);
+
+static NTSTATUS wb_irpc_GetDCName(struct irpc_message *msg,
+				  struct wbint_DsGetDcName *req)
+{
+
+	struct tevent_req *subreq = NULL;
+	struct wb_irpc_GetDCName_state *state = NULL;
+
+	state = talloc_zero(msg, struct wb_irpc_GetDCName_state);
+	if (state == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	state->msg = msg;
+	state->req = req;
+
+	subreq = wb_dsgetdcname_send(msg,
+				     global_event_context(),
+				     req->in.domain_name,
+				     req->in.domain_guid,
+				     req->in.site_name,
+				     req->in.flags);
+	if (subreq == NULL) {
+		return NT_STATUS_NO_MEMORY;
+	}
+
+	tevent_req_set_callback(subreq,
+				wb_irpc_GetDCName_done,
+				state);
+
+	msg->defer_reply = true;
+
+	return NT_STATUS_OK;
+}
+
+static void wb_irpc_GetDCName_done(struct tevent_req *subreq)
+{
+	struct wb_irpc_GetDCName_state *state = tevent_req_callback_data(
+		subreq, struct wb_irpc_GetDCName_state);
+	NTSTATUS status;
+
+	status = wb_dsgetdcname_recv(subreq, state->msg,
+				     state->req->out.dc_info);
+	TALLOC_FREE(subreq);
+	if (!NT_STATUS_IS_OK(status)) {
+		DBG_INFO("RPC callback failed for %s - %s\n", "DSGETDCNAME",
+			 nt_errstr(status));
+	}
+
+	state->req->out.result = status;
+
+	irpc_send_reply(state->msg, NT_STATUS_OK);
+}
+
 NTSTATUS wb_irpc_register(void)
 {
 	NTSTATUS status;
@@ -772,6 +837,12 @@ NTSTATUS wb_irpc_register(void)
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
+	status = IRPC_REGISTER(winbind_imessaging_context(),
+			       winbind, WBINT_DSGETDCNAME,
+			       wb_irpc_GetDCName, NULL);
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
 
 	return NT_STATUS_OK;
 }
